﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;

namespace RibbonTest
{
    /// <summary>
    /// View-Layer: Binding-List für Aktivitäten
    /// </summary>
    public class ActivityBindingList : BindingList<Activity>
    {
        /// <summary>
        /// Collections für das Change-Tracking
        /// </summary>
        private ActivityContainer inserts = new ActivityContainer();
        private ActivityContainer deletes = new ActivityContainer();

        /// <summary>
        /// Konstruktor. Fügt die Aktivitäten in diese Bindinglist ein
        /// </summary>
        /// <param name="projects">Liste der Aktivitäten</param>
        public ActivityBindingList( ActivityContainer activities )
            :this( activities.ToList<Activity>() )
        {
        }

        /// <summary>
        /// Konstruktor. Fügt die Aktivitäten in diese Bindinglist ein
        /// </summary>
        /// <param name="projects">Liste der Aktivitäten</param>
        public ActivityBindingList( List<Activity> activities )
        {
            // Aktivitäten in die Bindinglist übernehmen
            foreach ( Activity act in activities )
                this.Add( act );

            this.ListChanged += new ListChangedEventHandler( ActivityBindingList_ListChanged );
        }

        /// <summary>
        /// Change-Tracking für neue Objekte.
        /// </summary>
        /// <remarks>Das Change-Tracking für gelöschte Objekte wird separat behandelt, da 
        /// das Element beim Aufruf dieser Funktion bereits aus der Liste entfernt ist</remarks>
        /// <param name="sender">Dieses Objekt</param>
        /// <param name="e">Informationen über das geänderte Listenelement</param>
        void ActivityBindingList_ListChanged( object sender, ListChangedEventArgs e )
        {
            // Neues Element: in Liste der neuen Aktivitäten einfügen
            if ( e.ListChangedType == ListChangedType.ItemAdded )
                this.inserts.Add( this[e.NewIndex] );
        }

        /// <summary>
        /// Change-Tracking für gelöschte Objekte
        /// </summary>
        /// <param name="index">Index des Objektes das gelöscht wird.</param>
        protected override void RemoveItem( int index )
        {
            Activity deletedActivity = this[index];

            // wenn das Element bereits existierte (d.h. es ist nicht in der Inserts-Liste), dann in Lösch-Liste eintragen
            if ( !this.inserts.Contains( deletedActivity ) )
                this.deletes.Add( this[index] );

            // ggf. aus Einfüge- entfernen
            this.inserts.Remove( deletedActivity );

            base.RemoveItem( index );
        }


    }

}
